
# ********************************************************************************
#
# This file has been auto-generated by a code generation tool.
# Version:   4.2.11
# Date/time: 01 May 2025 14:41:44
# Template:  templates/pythonpws/version.razor.
#
#
#  Copyright (c) DNV AS. All rights reserved.
#
#  This source code is subject to copyright protection under international
#  copyright laws.  Reproduction, disclosure, distribution or use of this source
#  code is prohibited without the express written permission of DNV AS.
#
# ********************************************************************************

__version__ = "4.2.11"